package com.jiuqi.sql;

import java.util.Date;

public abstract class InputDataSet
{
    private boolean active;
    private FieldInfoList fieldInfos;
    private InputRecordBuffer buffer;
    private DataSetListener listener;
    
    public void resetListener(final DataSetListener listerner) {
        this.listener = listerner;
    }
    
    private FieldInfo findField(final String fieldName) throws DBException {
        final int index = this.fieldInfos.find(fieldName);
        if (index < 0) {
            throw new DBException(String.valueOf(fieldName) + " Not Exists");
        }
        final FieldInfo result = this.fieldInfos.get(index);
        return result;
    }
    
    public abstract InputDataSetMetaData getMetaData() throws DBException;
    
    protected abstract boolean getNextRecord(final InputRecordBuffer p0) throws DBException;
    
    protected abstract void initFields(final FieldInfoList p0) throws DBException;
    
    protected abstract InputRecordBuffer internalOpen() throws DBException;
    
    protected abstract void internalClose() throws DBException;
    
    public InputDataSet() {
        this.listener = null;
        this.fieldInfos = new FieldInfoList();
    }
    
    protected InputRecordBuffer recordBuffer() {
        return this.buffer;
    }
    
    public void open() throws DBException {
        if (!this.isActive()) {
            this.initFields(this.fieldInfos);
            this.buffer = this.internalOpen();
            this.fieldInfos.refreshFinder();
            this.active = true;
            if (this.listener != null) {
                this.listener.afterEvent(new DataSetEvent(this, 1));
            }
        }
    }
    
    public void first() throws DBException {
        throw new DBException("尚未实现！");
    }
    
    public boolean isActive() {
        return this.active;
    }
    
    public boolean next() throws DBException {
        boolean result = false;
        do {
            result = this.getNextRecord(this.buffer);
        } while (result && this.buffer.isDeleted());
        return result;
    }
    
    public void close() throws DBException {
        if (this.isActive()) {
            this.internalClose();
            this.active = false;
            if (this.listener != null) {
                this.listener.afterEvent(new DataSetEvent(this, 2));
            }
        }
    }
    
    public int fieldIndexOf(final String fieldName) {
        final int result = this.fieldInfos.find(fieldName);
        return (result < 0) ? result : (result + 1);
    }
    
    public boolean getBoolean(final int index) throws DBException {
        return this.buffer.readBoolean(this.fieldInfos.get(index - 1));
    }
    
    public boolean getBoolean(final String fieldName) throws DBException {
        return this.buffer.readBoolean(this.findField(fieldName));
    }
    
    public int getInt(final int index) throws DBException {
        return this.buffer.readInt(this.fieldInfos.get(index - 1));
    }
    
    public int getInt(final String fieldName) throws DBException {
        return this.buffer.readInt(this.findField(fieldName));
    }
    
    public long getLong(final int index) throws DBException {
        return this.buffer.readLong(this.fieldInfos.get(index - 1));
    }
    
    public long getLong(final String fieldName) throws DBException {
        return this.buffer.readLong(this.findField(fieldName));
    }
    
    public String getString(final int index) throws DBException {
        return this.buffer.readString(this.fieldInfos.get(index - 1));
    }
    
    public String getString(final String fieldName) throws DBException {
        return this.buffer.readString(this.findField(fieldName));
    }
    
    public double getDouble(final int index) throws DBException {
        return this.buffer.readDouble(this.fieldInfos.get(index - 1));
    }
    
    public double getDouble(final String fieldName) throws DBException {
        return this.buffer.readDouble(this.findField(fieldName));
    }
    
    @Deprecated
    public Date readDate(final int index) throws DBException {
        return this.buffer.readDate(this.fieldInfos.get(index - 1));
    }
    
    @Deprecated
    public Date readDate(final String fieldName) throws DBException {
        return this.buffer.readDate(this.findField(fieldName));
    }
    
    public Date getDate(final int index) throws DBException {
        return this.buffer.readDate(this.fieldInfos.get(index - 1));
    }
    
    public Date getDate(final String fieldName) throws DBException {
        return this.buffer.readDate(this.findField(fieldName));
    }
    
    private Object getObject(final FieldInfo fieldInfo) throws DBException {
        switch (fieldInfo.type) {
            case 7:
            case 8: {
                final long value = this.buffer.readLong(fieldInfo);
                return this.buffer.wasNull() ? null : new Long(value);
            }
            case 9: {
                return null;
            }
            case 1: {
                final boolean value2 = this.buffer.readBoolean(fieldInfo);
                return this.buffer.wasNull() ? null : new Boolean(value2);
            }
            case 2: {
                final Date value3 = this.buffer.readDate(fieldInfo);
                return this.buffer.wasNull() ? null : value3;
            }
            case 3: {
                final double value4 = this.buffer.readDouble(fieldInfo);
                return this.buffer.wasNull() ? null : new Double(value4);
            }
            case 5: {
                final int value5 = this.buffer.readInt(fieldInfo);
                return this.buffer.wasNull() ? null : new Integer(value5);
            }
            case 6: {
                final String value6 = this.buffer.readString(fieldInfo);
                return this.buffer.wasNull() ? null : value6;
            }
            default: {
                return null;
            }
        }
    }
    
    public final Object getObject(final int index) throws DBException {
        return this.getObject(this.fieldInfos.get(index - 1));
    }
    
    public final Object getObject(final String fieldName) throws DBException {
        return this.getObject(this.findField(fieldName));
    }
    
    public boolean wasNull() throws DBException {
        return this.buffer.wasNull();
    }
}
